/*
  File: OTGString.h
  Authors: Eric Bergerson
  Modification History:
  
  Created 8/16/90
  
  8/19/90 - Added -findString using code that 
  was originally strStr() by Alex Cone - Eric Bergerson

  Copyright (c) 1990 Objective Technologies, Inc.
  All Rights Reserved.
*/

#define STRING_CURRENT_VERSION 10

#import <appkit/appkit.h>

@interface OTGString : Object 
  
  /*
   * A String object performs dynamic allocation 
   * of memory to manage a character buffer.  It also
   * provides all of the standard string manipulation
   * functionality found in the standard C libraries. 
   */
  
{ 
  char *string; 
}

/* 
 * METHOD TYPES:
 * Creating and freeing a String
 * Copying a String
 * Getting and Allocating the String
 * Concatenating to the String
 * Comparing to the String
 * String Characteristics
 * Locating Parts of the String
 * Archiving
 */

+ initialize;
/*
 * TYPE; Creating; Sets the current version number
 *
 * Sets the current version number.
 */

- init;
/*
 * TYPE: Creating; Returns a new String object
 * 
 * Creates a new String object.  The string is empty;
 */

- init: (const char *) aString;
/*
 * TYPE: Creating; Returns a new String object containing aString
 * 
 * Creates a new String object.  The object will null terminated the
 * string aString.  The string buffer is resized to accomadate aString.
 */

- free;
/*
 * TYPE: Creating; Releases storage used by the String instance
 * 
 * Releases storage used by the String instance.
 */

- freeString;
/*
 * TYPE: Creating; Releases storage used by the string buffer
 * 
 * Releases storage used by the string buffer.  The string is set
 * to NULL.
 */

- (const char *) string;
/* 
 * TYPE: Getting; Returns the string pointer
 * 
 * Returns the string pointer.
 */

- (const char *) stringValue;
/* 
 * TYPE: Getting; Returns the string pointer
 * 
 * Returns the string pointer.  This method is the same as the
 * string method.  It was included to make the object compatible
 * with the similar methods of the same name in the appkit.
 */

- (const char *) validString;
/*
 * TYPE: Getting; Returns a valid string pointer or an empty string
 *
 * Returns the string pointer.  If the string pointer is NULL a 
 * pointer to an empty string (one byte containing a null) will be 
 * returned.
 */

- (char *) string: (size_t) size;
/* 
 * TYPE: Getting; Resizes the storage space and returns the string pointer
 * 
 * Resizes the storage space to size bytes and returns the string 
 * pointer.  If the new size is smaller than the old size the returned
 * string will be truncated and null terminated.
 */


- (BOOL) allocate: (size_t) size;
/*
 * TYPE: Getting; Allocates the string buffer to size bytes
 *
 * Re-allocates the string buffer to size bytes.  If the new size is 
 * smaller than the old size the string will be truncated and null 
 * terminated. Returns YES if successful and NO if the buffer cannot
 * be resized.
 */

- (size_t) size;
/* 
 * TYPE: Getting; Returns the size of the allocated string buffer
 *
 * Returns the size of the allocated string buffer.
 */

- (const char *) strcat: (const char *) aString;
/*
 * TYPE: Concatenating; Concatenates aString to the string
 * 
 * Concatenates aString to the string. Returns a null terminated string.
 * The string buffer is resized to accomadate aString.
 */

- (const char *) strncat: (const char *) aString for: (int) n_chars;
/*
 * TYPE: Concatenating; Concatenates at most n_chars from aString to the string.
 * 
 * Concatenates at most n_chars from aString to the string. Returns a 
 * null terminated string. The string buffer is resized to accomadate 
 * aString.
 */
  
- (int) strcmp: (const char *) aString;
/*
 * TYPE: Comparing; Compares aString to the string. 
 *
 * Compares aString to the string.  "Returns an integer 
 *  greater than, equal to, or less than 0, according as s1 is 
 * lexico-graphically greater than, equal to, or less than s2." (The 
 * strcmp(3) manual page).
 */

- (int) strncmp: (const char *) aString for: (int) n_chars;
/*
 * TYPE: Comparing; Compares at most n_chars of aString to the string. 
 *
 * Compares at most n_chars of aString to the string. Return values are
 * the same as strcmp.
 */

- (const char *) strcpy: (const char *) aString;
/*
 * TYPE: Copying; Copies aString to the string
 * 
 * Copies aString to the string. Returns a null terminated string.
 * The string buffer is resized to accomadate aString.
 */

- (const char *) strncpy: (const char *) aString for: (int) n_chars;
/*
 * TYPE: Copying; Copies at most n_chars from aString to the string.
 * 
 * Copies at most n_chars from aString to the string. Returns a 
 * null terminated string.  The string buffer is resized to accomadate 
 * aString.
 */

- (int) strlen;
/*
 * TYPE: String; Returns the lenght of the string
 *
 * Returns the lenght of the string.
 */

- (const char *) index: (char) c;
/*
 * TYPE: Locating; Locates the first c in the string;
 *
 * Returns the first occurrence of character c in the string.  Returns
 * NULL if c does not occur in the string.
 */
  
- (const char *) rindex: (char) c;
/*
 * TYPE: Locating; Locates the last c in the string;
 *
 * Returns the last occurrence of character c in the string.  Returns
 * NULL if c does not occur in the string.
 */
  
- (const char *) findString: (const char *) aString;
/*
  * TYPE: Locating; Locates the first occurence of aString in the string
  * 
  * Returns a pointer to the first occurence of aString in 
  * the string.  Returns NULL if aString can not be found in 
  * the string.
  */


- (BOOL) isEmpty;
/* 
 * TYPE: String; Returns YES if the string is NULL or only white space
 *
 * Returns YES if the string is NULL or contains only white space.  
 * White space is defined to be a space, tab, or return.
 */

- write: (NXTypedStream *) stream;
/*
 * TYPE: Archiving; Writes the String object to the typed stream
 *
 * Writes the receiving String to the typed stream stream.
 */

- read: (NXTypedStream *) stream;
/*
 * TYPE: Archiving; Reads the String object from the typed stream
 *
 * Reads in the String from the typed stream stream.
 */

@end
